﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Threading;
using System.Runtime.InteropServices;       ///< To use Toradex Dlls.

namespace vcsharp_glcd_demo
{
    public partial class Form1 : Form
    {
        [DllImport("GPIOLib.dll", CharSet = CharSet.Auto)]                              ///< Importing DLL.
        public static extern bool SetPinAltFn(int pinNum, int altFn, bool dirOut);      ///< External function declaration.

        [DllImport("GPIOLib.dll", CharSet = CharSet.Auto)]
        public static extern void InitGPIOLib();

        [DllImport("GPIOLib.dll", CharSet = CharSet.Auto)]
        public static extern bool SetPinLevel(int gpioNum, int val);

        [DllImport("GPIOLib.dll", CharSet = CharSet.Auto)]
        public static extern bool GetPinLevel(int gpioNum);

        [DllImport("GPIOLib.dll", CharSet = CharSet.Auto)]
        public static extern void DeInitGPIOLib();

        /// GPIOs used for GLCD
        private const int GLCD_RW =     28;          ///<  PWM_B.
        private const int GLCD_RS =     30;          ///<  PWM_C.
        private const int GLCD_RST =    67;          ///<  PWM_D.
        private const int GLCD_CS =     86;          ///<  SSP_FRM.
        private const int GLCD_C86 =    88;          ///<  SSP_SCLK.
        private const int GLCD_LEDA =   92;          ///<  SSP_TXD.
        private const int GLCD_EN =     90;          ///<  SSP_RXD.
        private const int GLCD_DATA7 =  98;          ///<  GPIO_98.
        private const int GLCD_DATA6 =  133;         ///<  GPIO_133.
        private const int GLCD_DATA5 =  103;         ///<  GPIO_103.
        private const int GLCD_DATA4 =  101;         ///<  GPIO_101.
        private const int GLCD_DATA3 =  97;          ///<  GPIO_97.
        private const int GLCD_DATA2 =  85;          ///<  GPIO_85.
        private const int GLCD_DATA1 =  79;          ///<  GPIO_79.
        private const int GLCD_DATA0 =  45;          ///<  GPIO_45.
                                                 
        private const int GLCD_COL1 =   1;
        private const int GLCD_ON =     0xAE;
        private const int GLCD_OFF =    0xAF;
        private const int GLCD_PAGE1 =  0xB0;
        private const int GLCD_PAGE2 =  0xB1;
        private const int GLCD_PAGE3 =  0xB2;
        private const int GLCD_PAGE4 =  0xB3;
        private const int GLCD_PAGE5 =  0xB4;
        private const int GLCD_PAGE6 =  0xB5;
        private const int GLCD_PAGE7 =  0xB6;
        private const int GLCD_PAGE8 =  0xB7;

        private const bool DIR_OUT = true;
        private const int GPIO = -1;
        private const int HIGH = 1;
        private const int LOW = 0;
        private const int DELAY = 1;
        private const int MONOCHROME = 1;
        private const int LCDWIDTH = 128;
        private const int LCDHEIGHT = 64;
        private const int LCDTOTALBYTES = 128 * 64 / 8;
        private byte[] glcdDataPin = new byte[]{GLCD_DATA0, GLCD_DATA1, GLCD_DATA2, GLCD_DATA3, GLCD_DATA4, GLCD_DATA5, GLCD_DATA6, GLCD_DATA7};
        private byte[] fileContent = new byte[LCDTOTALBYTES];
        private byte[] glcdData = new byte[LCDTOTALBYTES];                ///< Buffer of pixel values in GLCD format.
        private byte[] bitsBuffer = new byte[LCDWIDTH * LCDHEIGHT];       ///< Buffer to store unpacked bits from inputArray (1024 bytes x 8 Bits).
        private FileStream fileStream = null;
        private BinaryReader fileReader = null;

        private void GlcdConfig()
        {
            /// Configures GPIOs as output.
            SetPinAltFn(GLCD_EN, GPIO, DIR_OUT);
            SetPinAltFn(GLCD_RW, GPIO, DIR_OUT);
            SetPinAltFn(GLCD_RS, GPIO, DIR_OUT);
            SetPinAltFn(GLCD_RST, GPIO, DIR_OUT);
            SetPinAltFn(GLCD_CS, GPIO, DIR_OUT);
            SetPinAltFn(GLCD_C86, GPIO, DIR_OUT);
            SetPinAltFn(GLCD_DATA0, GPIO, DIR_OUT);
            SetPinAltFn(GLCD_DATA1, GPIO, DIR_OUT);
            SetPinAltFn(GLCD_DATA2, GPIO, DIR_OUT);
            SetPinAltFn(GLCD_DATA3, GPIO, DIR_OUT);
            SetPinAltFn(GLCD_DATA4, GPIO, DIR_OUT);
            SetPinAltFn(GLCD_DATA5, GPIO, DIR_OUT);
            SetPinAltFn(GLCD_DATA6, GPIO, DIR_OUT);
            SetPinAltFn(GLCD_DATA7, GPIO, DIR_OUT);
            SetPinAltFn(GLCD_LEDA, GPIO, DIR_OUT);
            /// Sets default GPIO pin Level as OFF.
            SetPinLevel(GLCD_EN, LOW);
            SetPinLevel(GLCD_RW, LOW);
            SetPinLevel(GLCD_RS, LOW);
            SetPinLevel(GLCD_RST, LOW);
            SetPinLevel(GLCD_CS, LOW);
            SetPinLevel(GLCD_C86, LOW);
            SetPinLevel(GLCD_DATA0, LOW);
            SetPinLevel(GLCD_DATA1, LOW);
            SetPinLevel(GLCD_DATA2, LOW);
            SetPinLevel(GLCD_DATA3, LOW);
            SetPinLevel(GLCD_DATA4, LOW);
            SetPinLevel(GLCD_DATA5, LOW);
            SetPinLevel(GLCD_DATA6, LOW);
            SetPinLevel(GLCD_DATA7, LOW);
            SetPinLevel(GLCD_LEDA, LOW);
        }

        private void GlcdSendDataByte(byte glcdDataByte)
        {
            byte glcdBitCounter = 0;
            byte glcdDataBit = 0;
            byte glcdDataSave = glcdDataByte;
            /// Sets data pins according to data to send.
            for (glcdBitCounter = 0; glcdBitCounter < 8; glcdBitCounter++)
            {
                glcdDataByte = glcdDataSave;
                glcdDataBit = (byte)((glcdDataByte >> glcdBitCounter) & 0x01);
                SetPinLevel(glcdDataPin[glcdBitCounter], glcdDataBit);
            }
        }

        private void GlcdWriteCmd(byte glcdCmd)
        {
            SetPinLevel(GLCD_RS, LOW);
            GlcdDelay(DELAY);
            SetPinLevel(GLCD_CS, LOW);
            GlcdDelay(DELAY);
            SetPinLevel(GLCD_RW, LOW);
            GlcdDelay(DELAY);
            /// Sends command to GLCD.
            GlcdSendDataByte(glcdCmd);
            GlcdDelay(DELAY);
            SetPinLevel(GLCD_RW, HIGH);
            GlcdDelay(DELAY);
            SetPinLevel(GLCD_CS, HIGH);
            GlcdDelay(DELAY);
            SetPinLevel(GLCD_RS, HIGH);
            GlcdDelay(DELAY);
        }

        void GlcdWriteData(byte glcdData)
        {
            SetPinLevel(GLCD_RS, HIGH);
            GlcdDelay(DELAY);
            SetPinLevel(GLCD_CS, LOW);
            GlcdDelay(DELAY);
            SetPinLevel(GLCD_RW, LOW);
            GlcdDelay(DELAY);
            /// Sends data to GLCD.
            GlcdSendDataByte(glcdData);
            GlcdDelay(DELAY);
            SetPinLevel(GLCD_RW, HIGH);
            GlcdDelay(DELAY);
            SetPinLevel(GLCD_CS, HIGH);
            GlcdDelay(DELAY);
            SetPinLevel(GLCD_RS, LOW);
            GlcdDelay(DELAY);
        }

        private void GlcdSetPage(byte glcdPageNum, byte glcdColumnNum)
        {
            byte glcdMsb = 0;
            byte glcdLsb = 0;
            glcdMsb = (byte)(((glcdColumnNum & 0xF0) >> 4) | 0x10);
            glcdLsb = (byte)(glcdColumnNum & 0x0F);
            GlcdWriteCmd(glcdPageNum);
            GlcdWriteCmd(glcdMsb);
            GlcdWriteCmd(glcdLsb);
        }

        private void GlcdClear()
        {
            byte glcdPage = 0;
            byte glcdColumn = 0;

            GlcdSetPage(GLCD_PAGE1, 0);                     ///< Initial page and column.
            for (glcdPage = 0; glcdPage < 8; glcdPage++)
            {
                for (glcdColumn = 0; glcdColumn < 128; glcdColumn++)
                {
                    GlcdWriteData(0x00);
                }
                GlcdSetPage((byte)(GLCD_PAGE1 + glcdPage + 1), 0);    ///< Increments page.
            }
            GlcdSetPage(GLCD_PAGE1, 0);                     ///< Initial page and column.
        }

        private void GlcdBackLight(byte glcdLedStatus)
        {
            /// Sets/Resets Back-light control pin.
            SetPinLevel(GLCD_LEDA, glcdLedStatus);
        }

        private void GlcdDelay(int delay)
        {
            int delayCount = 0;
            while (delay > 0)
            {
                delayCount = 10;
                while (delayCount > 0)
                {
                    delayCount--;
                }
                delay--;
            }
        }

        private void GlcdInit()
        {
            GlcdConfig();
            Thread.Sleep(1);
            SetPinLevel(GLCD_C86, LOW);         ///< Selects Parallel Data Mode.
            SetPinLevel(GLCD_EN, HIGH);
            Thread.Sleep(10);
            SetPinLevel(GLCD_RST, LOW);         ///< Clears Reset Pin.
            Thread.Sleep(1);     
            SetPinLevel(GLCD_RST, HIGH);        ///< Sets Reset Pin.
            GlcdWriteCmd(0xE2);                 ///< Software Reset GLCD.
            GlcdDelay(DELAY);  

            /// Sends Initialization commands to GLCD.
            GlcdWriteCmd(0xA0);
            GlcdDelay(DELAY);
            GlcdWriteCmd(0xC8);
            GlcdDelay(DELAY);
            GlcdWriteCmd(0xA3);
            GlcdDelay(DELAY);
            GlcdWriteCmd(0x2F);
            GlcdDelay(DELAY);
            GlcdWriteCmd(0x21);
            GlcdDelay(DELAY);
            GlcdWriteCmd(0x81);
            GlcdDelay(DELAY);
            GlcdWriteCmd(0x40);
            GlcdDelay(DELAY);
            GlcdWriteCmd(0xA4);
            GlcdDelay(DELAY);
            GlcdWriteCmd(0xA6);
            GlcdDelay(DELAY);
            GlcdWriteCmd(0x81);
            GlcdDelay(DELAY);
            GlcdWriteCmd(0x3F);
            GlcdDelay(DELAY);
            GlcdWriteCmd(0xAF);
            GlcdDelay(DELAY);
            GlcdWriteCmd(0xB0);
            GlcdDelay(DELAY);
            GlcdWriteCmd(0x10);
            GlcdDelay(DELAY);
            GlcdWriteCmd(0x00);
            GlcdDelay(DELAY);
        }

        /// Opens a bmp file. Sets fileExist parameter to false and returns false if file not found.
        /// Sets fileExist parameter to true and returns false if file doesn't has required format.
        private bool OpenBmpFile(string path, ref bool fileExist)
        {
            byte[] readData = new byte[4];
            string readDataStringFormat = null;
            int dataOffset = 0;
            int imageParameters = 0;

            fileReader = null;
            fileStream = null;
            fileExist = false;
            try
            {
                fileStream = File.Open(path, FileMode.Open);
            }
            catch
            {
                return false;
            }
            fileExist = true;
            try
            {
                fileReader = new BinaryReader(fileStream);
                /// Checks weather, file is correct bitmap image or not.
                fileStream.Seek(0, SeekOrigin.Begin);
                readData = fileReader.ReadBytes(2);
                readDataStringFormat = Encoding.UTF8.GetString(readData, 0, 2);
                if (readDataStringFormat != "BM")
                {
                    throw new Exception();
                }
                /// Gets offset of bitmap data from the beginning of the file.
                fileStream.Seek(10, SeekOrigin.Begin);
                readData = fileReader.ReadBytes(4);
                dataOffset = BitConverter.ToInt32(readData, 0);
                /// Gets Horizontal width of the image.
                fileStream.Seek(18, SeekOrigin.Begin);
                readData = fileReader.ReadBytes(4);
                imageParameters = BitConverter.ToInt32(readData, 0);
                if (imageParameters != LCDWIDTH)
                {
                    throw new Exception();
                }
                /// Gets Vertical height of the image.
                readData = fileReader.ReadBytes(4);
                imageParameters = BitConverter.ToInt32(readData, 0);
                if (imageParameters != LCDHEIGHT)
                {
                    throw new Exception();
                }
                Array.Clear(readData, 0, readData.Length);
                /// Gets the number of layers in the bitmap image.
                readData = fileReader.ReadBytes(2);
                imageParameters = BitConverter.ToInt16(readData, 0);
                if (imageParameters != 1)               ///< 1 = Black and white layer.
                {
                    throw new Exception();
                }
                /// Gets bits per pixel (BPP) in the bitmap image.
                readData = fileReader.ReadBytes(2);
                imageParameters = BitConverter.ToInt16(readData, 0);
                if (imageParameters != MONOCHROME)
                {
                    throw new Exception();
                }
                /// Gets the type of compression done on the bitmap image.
                readData = fileReader.ReadBytes(4);
                imageParameters = BitConverter.ToInt32(readData, 0);
                if (imageParameters != 0)               ///< 0 = No compression.
                {
                    throw new Exception();
                }
                /// Gets pixel values in-to the buffer.
                fileStream.Seek(dataOffset, SeekOrigin.Begin);
                fileContent = fileReader.ReadBytes(LCDTOTALBYTES);
            }
            catch
            {
                fileReader.Close();
                fileStream.Close();
                return false;
            }
            fileReader.Close();
            fileStream.Close();
            readData = null;
            return true;
        }

        /// Sends BMP file content to Glcd to display.
        private void BmpToGlcd()
        {
            int glcdPage = 0;
            int bitNumber = 0;
            int count1 = 0;
            int count2 = 0;
            int count3 = 0;
            int glcdColumn = 0;
            int columnNumber = 0;
            int glcdImageArray = 0;
            int tempRasterScanNumber = 0;

            Array.Clear(glcdData, 0, glcdData.Length);
            Array.Clear(bitsBuffer, 0, bitsBuffer.Length);
            /// Converts data from file, into bits (Raster scanning).
            for (tempRasterScanNumber = 896; tempRasterScanNumber >= 0; tempRasterScanNumber = tempRasterScanNumber - 128)
            {
	            for (count1 = 0; count1 < 128; count1++)
	            {
		            for (bitNumber = 0; bitNumber < 8; bitNumber++)
		            {
			            bitsBuffer[count3] = (byte)(fileContent[tempRasterScanNumber + count1] & ((0x80) >> bitNumber));
			            if (bitsBuffer[count3] > 0)
				            bitsBuffer[count3] = 1;
			            else
				            bitsBuffer[count3] = 0;
			            count3++;
		            }
	            }
            }
            /// Converts bits buffer into GLCD format.
            count1 = 0;
            for (count1 = 0; count1 < 1024; count1++)
            {
	            if ((count1 % 128 == 0) && (count1 != 0))
	            {
		            count2 = count2 + 1024;
		            columnNumber = 0;
	            }
	            for (bitNumber = 0; bitNumber < 8; bitNumber++)
	            {  
		            glcdData[count1] = (byte)(glcdData[count1] | ((bitsBuffer[(count2 + columnNumber + (bitNumber * 128))] & 0x01) << (7- bitNumber)));
	            }
	            columnNumber++;
            }
            /// Displays image on GLCD.
            GlcdSetPage(GLCD_PAGE1, 0);                  ///< Initial page of GLCD.
            for (glcdPage = 0; glcdPage < 8; glcdPage++)
            {
                for (glcdColumn = 0; glcdColumn < 128; glcdColumn++)
                {
                    GlcdWriteData(glcdData[glcdImageArray++]);
                }
                GlcdSetPage((byte)(GLCD_PAGE1 + glcdPage + 1), 0);   ///< Increments GLCD page.
            }
        }

        public Form1()
        {
            InitializeComponent();
            InitGPIOLib();                          ///< Initializes GPIO library.
            GlcdInit();
            GlcdClear();                            ///< Clears the Glcd.
            pictureBox1.SizeMode = PictureBoxSizeMode.StretchImage;
        }

        private void displayButton_Click(object sender, EventArgs e)
        {
            if (displayButton.Text == "Start")
            {
                textBox1.BringToFront();
                GlcdBackLight(HIGH);                ///< Switch ON back-light.
                displayButton.Text = "Stop";
                statusBox.Text = "Displaying...";
                updateDisplayTimer.Enabled = true;
            }
            else
            {
                updateDisplayTimer.Enabled = false;
                GlcdBackLight(LOW);                 ///< Switch OFF back-light.
                pictureBox1.Image = null;
                displayButton.Text = "Start";
                statusBox.Text = "Stopped.";
                GlcdClear();
            }
        }

        private void Form1_Closing(object sender, CancelEventArgs e)
        {
            if (displayButton.Text == "Stop")
            {
                displayButton_Click(sender, e);
            }
            DeInitGPIOLib();                        ///< De-initializes GPIO library.
        }

        private void updateDisplayTimer_Tick(object sender, EventArgs e)
        {
            int imageCount = 0;
            bool retVal = false;
            bool fileExist = false;
            string finalPath = null;
            Bitmap bitmap = null;

            for (imageCount = 1; imageCount <= 11; imageCount++)
            {
                finalPath = "\\Program Files\\vcsharp_glcd_demo\\images\\horsepicture_" + imageCount.ToString() + ".bmp";
                retVal = OpenBmpFile(finalPath, ref fileExist);
                if (retVal == false)
                {
                    if (fileExist == false)
                    {
                        statusBox.Text = "Image not found.";
                    }
                    else
                    {
                        statusBox.Text = "Failed to display.";
                    }
                    if (displayButton.Text == "Stop")
                    {
                        displayButton_Click(sender, e);
                    }
                }
                else
                {
                    BmpToGlcd();
                    bitmap = null;
                    bitmap = new Bitmap(finalPath);
                    pictureBox1.Image = bitmap;
                    pictureBox1.BringToFront();
                    pictureBox1.Update();
                    Thread.Sleep(50);
                }
            }
        }
    }
}